<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/

class GOOGLELOCATION_BOL_LocationDao extends OW_BaseDao
{
    const ENTITY_ID = 'entityId';
    const ENTITY_TYPE = 'entityType';
    const COUNTRY_CODE = 'countryCode';
    const ADDRESS_STRING = 'address';
    const LATITUDE = 'lat';
    const LONDITUDE = 'lng';
    
    const ENTITY_TYPE_USER = 'user';
    const ENTITY_TYPE_EVENT = 'event';

    /**
     * Class constructor
     *
     */
    protected function __construct()
    {
        parent::__construct();
    }
    /**
     * Class instance
     *
     * @var GOOGLELOCATION_BOL_LocationDao
     */
    private static $classInstance;

    /**
     * Returns class instance
     *
     * @return GOOGLELOCATION_BOL_LocationDao
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * @see OW_BaseDao::getDtoClassName()
     *
     */
    public function getDtoClassName()
    {
        return 'GOOGLELOCATION_BOL_Location';
    }

    /**
     * @see OW_BaseDao::getTableName()
     *
     */
    public function getTableName()
    {
        return OW_DB_PREFIX . 'googlelocation_data';
    }
    
    /**
     * @param integer $userId
     * @return GOOGLELOCATION_BOL_Location
     */
    public function findByUserId( $userId )
    {
        if ( empty($userId) )
        {
            return null;
        }
        
        $example = new OW_Example();
        $example->andFieldEqual(self::ENTITY_ID, (int)$userId);
        $example->andFieldEqual(self::ENTITY_TYPE, self::ENTITY_TYPE_USER);
        return $this->findObjectByExample($example);
    }
    
    
    /**
     * @param array $userIdList
     * @return array <GOOGLELOCATION_BOL_Location>
     */
    public function findByUserIdList( array $userIdList )
    {
        if ( empty($userIdList) )
        {
            return array();
        }
        
        $example = new OW_Example();
        $example->andFieldInArray(self::ENTITY_ID, $userIdList);
        $example->andFieldEqual(self::ENTITY_TYPE, self::ENTITY_TYPE_USER);
        return $this->findListByExample($example);
    }


    /**
     * @param string $prefix
     * @param float $southWestLat
     * @param float $southWestLng
     * @param float $northEastLat
     * @param float $northEastLng
     * @param string $countryCode
     * @return string
     */
    
    public function getSearchInnerJoinSql( $prefix, $southWestLat, $southWestLng, $northEastLat, $northEastLng, $countryCode = null, $joinType = 'INNER' )
    {
        $countryStr = "";
        /*if ( !empty($countryCode) )
        {
            $countryStr =" AND location.".self::COUNTRY_CODE." = '".$this->dbo->escapeString($countryCode)."' ";
        }*/

        $sql = $joinType . " JOIN ".$this->getTableName()." location ON ( $prefix.id = location.entityId AND location.entityType = '".self::ENTITY_TYPE_USER."'
                 AND (
                         location.southWestLat >= ".(float)$southWestLat."
                         AND location.southWestLat <= ".(float)$northEastLat."
                             AND
                         location.northEastLat >= ".(float)$southWestLat."
                         AND location.northEastLat <= ".(float)$northEastLat."
                     )

                 AND (
                         location.southWestLng >= ".(float)$southWestLng."
                         AND location.southWestLng <= ".(float)$northEastLng."
                             AND
                         location.northEastLng >= ".(float)$southWestLng."
                         AND location.northEastLng <= ".(float)$northEastLng."
                     ) $countryStr ) ";

        return $sql;
    }
    
    /**
     * @return array
     */
    public function getAllLocationsForUserMap()
    {
        $queryParts = BOL_UserDao::getInstance()->getUserQueryFilter("u", "id", array(
            "method" => "GOOGLELOCATION_BOL_LocationDao::getAllLocationsForUserMap"
        ));

        $query = "SELECT l.* FROM " . $this->getTableName() . " l

                INNER JOIN `" . BOL_UserDao::getInstance()->getTableName() . "` as `u`
                    ON( `l`.`entityId` = `u`.`id` )

               " . $queryParts["join"] . "

                WHERE " . $queryParts["where"] . " AND l.entityType = '".self::ENTITY_TYPE_USER."' ";

        return $this->dbo->queryForList($query);
    }
    
    public function getAllLocationsForEntityType($entityType)
    {
        $query = "SELECT l.* FROM " . $this->getTableName() . " l
                WHERE l.entityType = :entityType ";
                
        return $this->dbo->queryForList($query, array('entityType' => $entityType));
    }
    
    public function findUserListByCoordinates( $lat, $lng, $first, $count, $userIdList = array() )
    {
        if ( !isset($lat) || !isset($lng) )
        {
            return array();
        }

        $where = "";

        if ( !empty($userIdList) && is_array($userIdList) )
        {
            $where = " AND u.id IN ( ". $this->dbo->mergeInClause($userIdList) ." ) ";
        }

        

        $queryParts = BOL_UserDao::getInstance()->getUserQueryFilter("u", "id", array(
            "method" => "GOOGLELOCATION_BOL_LocationDao::findUserListByCoordinates"
        ));

        $query = "SELECT u.* FROM " . $this->getTableName() . " l

                INNER JOIN `" . BOL_UserDao::getInstance()->getTableName() . "` as `u`
                    ON( `l`.`entityId` = `u`.`id` )

               " . $queryParts["join"] . "

                WHERE " . $queryParts["where"] . $where . " AND l.lat = :lat AND l.lng = :lng AND l.entityType = '".self::ENTITY_TYPE_USER."'  LIMIT :first, :count";
        
        return $this->dbo->queryForObjectList($query, BOL_UserDao::getInstance()->getDtoClassName(), array( 'lat' => $lat, 'lng' => $lng, 'first' => (int)$first, 'count' => (int)$count ));
    }
    
    public function findUserCountByCoordinates( $lat, $lng, $userIdList = array() )
    {
        if ( !isset($lat) || !isset($lng) )
        {
            return 0;
        }

        $where = "";

        if ( !empty($userIdList) && is_array($userIdList) )
        {
            $where = " AND u.id IN ( ". $this->dbo->mergeInClause($userIdList) ." ) ";
        }

        $queryParts = BOL_UserDao::getInstance()->getUserQueryFilter("u", "id", array(
            "method" => "GOOGLELOCATION_BOL_LocationDao::findUserCountByCoordinates"
        ));

        $query = "SELECT Count(DISTINCT l.entityId) FROM " . $this->getTableName() . " l

                INNER JOIN `" . BOL_UserDao::getInstance()->getTableName() . "` as `u`
                    ON( `l`.`entityId` = `u`.`id` )

                " . $queryParts["join"] . "

                WHERE " . $queryParts["where"] . $where . " AND l.lat = :lat AND l.lng = :lng AND l.entityType = '".self::ENTITY_TYPE_USER."' ";

        return $this->dbo->queryForColumn($query, array( 'lat' => $lat, 'lng' => $lng ));
    }
    
    public function getLocationName( $lat, $lng )
    {
        if ( !isset($lat) || !isset($lng) )
        {
            return null;
        }
        
        $query = "SELECT l.address FROM " . $this->getTableName() . " l
                WHERE l.lat = :lat AND l.lng = :lng LIMIT 1 ";
        
        return $this->dbo->queryForColumn($query, array( 'lat' => $lat, 'lng' => $lng ));
    }
    
    public function findEntityIdAndEntityType( $entityId, $entityType )
    {
        if ( empty($entityId) || empty($entityType) )
        {
            return null;
        }
        
        $example = new OW_Example();
        $example->andFieldEqual(self::ENTITY_ID, $entityId);
        $example->andFieldEqual(self::ENTITY_TYPE, $entityType);
        return $this->findObjectByExample($example);
    }
    
    public function deleteByEntityType( $entityType )
    {
        if ( empty($entityType) )
        {
            return false;
        }
        
        $example = new OW_Example();
        $example->andFieldEqual(self::ENTITY_TYPE, $entityType);
        return $this->deleteByExample($example);
    }
    
    public function deleteByEntityIdAndEntityType( $entityId, $entityType )
    {
        if ( empty($entityId) || empty($entityType) )
        {
            return false;
        }
        
        $example = new OW_Example();
        $example->andFieldEqual(self::ENTITY_ID, $entityId);
        $example->andFieldEqual(self::ENTITY_TYPE, $entityType);
        return $this->deleteByExample($example);
    }
    
    public function getListOrderedByDistance( $userIdList, $first, $count, $lat, $lon )
    {
        if ( empty($userIdList) )
        {
            return array();
        }
        
        $queryParts = BOL_UserDao::getInstance()->getUserQueryFilter("user", "id", array(
            "method" => "GOOGLELOCATION_BOL_LocationDao::getListOrderedByDistance"
        ));
        
        $where = '';
        
        $join = ' INNER JOIN `'.GOOGLELOCATION_BOL_LocationDao::getInstance()->getTableName(). '` location ON ( user.id = location.entityId AND location.entityType = \''.GOOGLELOCATION_BOL_LocationDao::ENTITY_TYPE_USER.'\' )';
            
        $sql = "SELECT `user`.* FROM `" . BOL_UserDao::getInstance()->getTableName() . "` `user`
            {$queryParts["join"]}
            LEFT JOIN `".GOOGLELOCATION_BOL_LocationDao::getInstance()->getTableName(). "` location ON ( user.id = location.entityId AND location.entityType = '".GOOGLELOCATION_BOL_LocationDao::ENTITY_TYPE_USER."' )
            WHERE `user`.`id` IN (" . $this->dbo->mergeInClause($userIdList) . ") $where
            ORDER BY if( location.id IS NULL, 99999999999999, (
            acos (
                    cos ( radians(:lat) )
                    * cos( radians( location.lat ) )
                    * cos( radians( location.lng ) - radians(:lon) )
                    + sin ( radians(:lat) )
                    * sin( radians( location.lat ) )
                ) )
            ) ASC , `user`.`activityStamp` DESC limit :first, :count";
            
        return $this->dbo->queryForObjectList($sql, $this->getDtoClassName(), array('first' => $first, 'count' => $count, 'lat' => $lat, 'lon' => $lon) );
    }
}
