<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/

class GOOGLELOCATION_CTRL_EventMap extends OW_ActionController
{
    const MAX_EVENT_COUNT = 16;
    
    private function getEventMapCmp($backUri = null)
    {
        if( !OW::getPluginManager()->isPluginActive('event') )
        {
            throw new Redirect404Exception();
        }
        
        $map = GOOGLELOCATION_BOL_LocationService::getInstance()->getMapComponent();
        $map->setHeight('600px');
        $map->setZoom(2);
        $map->setCenter(30,10);
        //$map->setMapOption('scrollwheel', 'false');

        $locationList = GOOGLELOCATION_BOL_LocationService::getInstance()->getAllLocationsForEntityType('event');

        $entityIdList = array();
        $entityLocationList = array();

        foreach( $locationList as $location )
        {
            $entityIdList[$location['entityId']] = $location['entityId'];
            $entityLocationList[$location['entityId']] = $location;
        }
        $locationList = $entityLocationList;
        
        $eventsList = EVENT_BOL_EventService::getInstance()->findPublicEvents(null, 1000);
        $publicEventsId = array();
        $tmpEventList = array();
        
        foreach( $eventsList as $event )
        {
            $publicEventsId[$event->id] = $event->id;
            $tmpEventList[$event->id] = $event;
        }
        $eventsList = $tmpEventList;
        
        $entityIdList = array_intersect($entityIdList, $publicEventsId);
         
        $publicLocationList = array();
        $publicEventList = array();
        
        foreach( $entityIdList as $entityId )
        {
            $publicLocationList[$entityId] = $locationList[$entityId];
            $publicEventList[$entityId] = $eventsList[$entityId];
        }
        
        $events = EVENT_BOL_EventService::getInstance()->getListingDataWithToolbar($publicEventList);
                
        $pointList = GOOGLELOCATION_BOL_LocationService::getInstance()->getPointList($publicLocationList);
        
        foreach( $pointList as $point )
        {
            if( !empty( $point['entityIdList'] ) )
            {
                $content = "";
                
                if ( $point['count'] > 1 ) 
                {
                    $listCmp = new GOOGLELOCATION_CMP_MapEventList($point['entityIdList'], $point['location']['lat'], $point['location']['lng'], $backUri);
                    $content .= $listCmp->render();
                    unset($listCmp);
                }
                else 
                {
                    $eventId = current($point['entityIdList']);
                    
                    if( !empty($events[$eventId]) )
                    {
                        $cmp = new GOOGLELOCATION_CMP_MapItem();
                        $cmp->setAvatar(array('src' => $events[$eventId]['imageSrc'] ));
                        $content = "<a href='{$events[$eventId]['eventUrl']}'>".$events[$eventId]['title']."</a>
                            <div>{$events[$eventId]['content']}</div>
                            <div>{$publicLocationList[$eventId]['address']}</div> ";
                        $cmp->setContent($content);

                        $content = $cmp->render();
                    }
                }
                
                if ( !empty($content) )
                {
                    $map->addPoint($point['location'], '', $content);
                }
            }
        }
        
        return $map;
    }
    
    public function map()
    {
        $event = new OW_Event('event.is_plugin_active');
        OW::getEventManager()->trigger($event);
        
        $data = $event->getData();
        
        if ( !$data )
        {
            throw new Redirect404Exception();
        }
        
        $event = new OW_Event('event.get_content_menu');
        OW::getEventManager()->trigger($event);
        
        $menu = $event->getData();
        
        $menu = EVENT_BOL_EventService::getInstance()->getContentMenu();
        $menu->getElement('events_map')->setActive(true);
        $this->addComponent('menu', $menu);

        $language = OW::getLanguage();
        $this->setPageHeading($language->text('googlelocation', 'map_page_heading'));
        $this->setPageHeadingIconClass('ow_ic_bookmark');
        
        $this->addComponent("map", $this->getEventMapCmp(OW::getRouter()->getUri()));
        
        OW::getEventManager()->trigger(new OW_Event('googlelocation.add_js_lib'));
    }
}
