/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***/

var MatchmakingCoefficient = function(params){
    this.checkedCoefficient = 0;
    this.cmpId = params.cmpId;
    this.itemsCount = params.itemsCount;
    this.id = params.id;
    
}

MatchmakingCoefficient.prototype = {
    init: function(){
        var self = this;
        
        for( var i = 1; i <= this.itemsCount; i++ ){
            $('#' + this.cmpId+'_item_'+ i).bind( 'mouseover', {i:i},
                function(e){
                    self.setCoefficient(e.data.i);
                }
            ).bind( 'mouseout',
                function(){
                    self.setCoefficient(self.checkedCoefficient);
                }
            ).bind( 'click', {i:i},
                function(e){
                    self.updateCoefficient(e.data.i);
                }
            );
        }
    },
    
    setCoefficient: function( coefficient ){
        for( var i = 1; i <= this.itemsCount; i++ ){
            var $el = $('#' + this.cmpId+'_item_'+ i);
            $el.removeClass('active');
            if( !coefficient ){
                continue;
            }
            if( i <= coefficient ){
                $el.addClass('active');
            }
        }
    },
    
    updateCoefficient: function( coefficient ){
        var self = this;
        
        self.checkedCoefficient = coefficient;        
        $('#'+self.id).val(coefficient);
        self.setCoefficient(coefficient);
    }
}

var EditCoefficient = function(params){

    this.checkedCoefficient = params.checkedCoefficient;
    this.cmpId = params.cmpId;
    this.itemsCount = params.itemsCount;
    this.id = params.id;
    this.respondUrl = params.respondUrl;

    this.updateCoefficient = function( coefficient )
    {
        var self = this;
        if( coefficient == this.checkedCoefficient ){
            return;
        }
        this.checkedCoefficientBackup = this.checkedCoefficient;
        this.checkedCoefficient = coefficient;
        $.ajax({
            type: 'POST',
            url: self.respondUrl,
            data: 'id='+encodeURIComponent(self.id)+'&coefficient='+encodeURIComponent(coefficient),
            dataType: 'json',
            success : function(data){

                if( data.errorMessage ){
                    OW.error(data.errorMessage);
                    self.checkedCoefficient = self.checkedCoefficientBackup;
                    self.setCoefficient(self.checkedCoefficientBackup);
                    return;
                }

                if( data.message ){
                    OW.info(data.message);
                }

                $('#'+self.id).val(coefficient);
                self.setCoefficient(coefficient);

            },
            error : function( XMLHttpRequest, textStatus, errorThrown ){
                alert('Ajax Error: '+textStatus+'!');
                throw textStatus;
            }
        });
    }
}

EditCoefficient.prototype = MatchmakingCoefficient.prototype;

var CreateCoefficient = function(params){

    this.checkedCoefficient = params.checkedCoefficient;
    this.cmpId = params.cmpId;
    this.itemsCount = params.itemsCount;
    this.name = params.name;
    this.respondUrl = params.respondUrl;

    this.updateCoefficient = function( coefficient ){
        var self = this;
        if( coefficient == this.checkedCoefficient ){
            return;
        }
        this.checkedCoefficientBackup = this.checkedCoefficient;
        this.checkedCoefficient = coefficient;
        $.ajax({
            type: 'POST',
            url: self.respondUrl,
            data: 'name='+encodeURIComponent(self.name)+'&coefficient='+encodeURIComponent(coefficient)+'&create=1',
            dataType: 'json',
            success : function(data){

                if( data.errorMessage ){
                    OW.error(data.errorMessage);
                    self.checkedCoefficient = self.checkedCoefficientBackup;
                    self.setCoefficient(self.checkedCoefficientBackup);
                    return;
                }

                if( data.message ){
                    OW.info(data.message);
                }

                $('#'+self.name).val(coefficient);
                self.setCoefficient(coefficient);

                location.reload(true);
            },
            error : function( XMLHttpRequest, textStatus, errorThrown ){
                alert('Ajax Error: '+textStatus+'!');
                throw textStatus;
            }
        });
    }
}

CreateCoefficient.prototype = MatchmakingCoefficient.prototype;

var MatchmakingRule = function(params){
    this.cmpId = params.cmpId;
    this.id = params.id;
    this.prefix = params.prefix;
}

MatchmakingRule.prototype = {
    init: function(){
        var self = this;

        $('#' + this.id).change(function(){
            if ($(':selected',this).val()=='')
            {
                return;
            }
            $('.match_question').val( self.prefix + ' ' + $(':selected',this).text() );
        });
    }
}

$(function(){
    $('#matchQuestionFormBtn').click(function(){
        if ($('#matchQuestionFormBtn').hasClass('ow_close_match_section')){
            $('#matchQuestionFormBtn').removeClass('ow_close_match_section');
            $('#matchQuestionForm').addClass('ow_hidden');
        }
        else{
            $('#matchQuestionForm').removeClass('ow_hidden');
            $('#matchQuestionFormBtn').addClass('ow_close_match_section');
        }
    });
})